@@ -6,6 +6,7 @@ from Selenium2Library import webdrivermonkeypatches
 from Selenium2Library.utils import BrowserCache
 from Selenium2Library.locators import WindowManager
 from keywordgroup import KeywordGroup
+from selenium.common.exceptions import NoSuchWindowException
 
 ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
 FIREFOX_PROFILE_DIR = os.path.join(ROOT_DIR, 'resources', 'firefoxprofile')
@@ -111,7 +112,13 @@ class _BrowserManagementKeywords(KeywordGroup):
             self._info("Opening browser '%s' to base url '%s'" % (browser, url))
         browser_name = browser
         browser = self._make_browser(browser_name,desired_capabilities,ff_profile_dir,remote_url)
-        browser.get(url)
+        try:
+            browser.get(url)
+        except:
+            self._cache.register(browser, alias)
+            self._debug("Opened browser with session id %s but failed to open url '%s'"
+                        % (browser.session_id, url))
+            raise
         self._debug('Opened browser with session id %s'
                     % browser.session_id)
         return self._cache.register(browser, alias)
@@ -148,11 +155,11 @@ class _BrowserManagementKeywords(KeywordGroup):
         | # use a proxy for PhantomJS |              |                                           |                         |
         | ${service args}=            | Create List  | --proxy=192.168.132.104:8888              |                         |
         | Create Webdriver            | PhantomJS    | service_args=${service args}              |                         |
-        
+
         Example for Robot Framework < 2.8:
-        | # debug IE driver |                   |                  |       |          |                       |
-        | ${kwargs}=        | Create Dictionary | log_level        | DEBUG | log_file | %{HOMEPATH}${/}ie.log |
-        | Create Webdriver  | Ie                | kwargs=${kwargs} |       |          |                       |
+        | # debug IE driver |                   |                  |                                | 
+        | ${kwargs}=        | Create Dictionary | log_level=DEBUG  | log_file=%{HOMEPATH}${/}ie.log |
+        | Create Webdriver  | Ie                | kwargs=${kwargs} |                                |
         """
         if not isinstance(kwargs, dict):
             raise RuntimeError("kwargs must be a dictionary.")
@@ -252,6 +259,26 @@ class _BrowserManagementKeywords(KeywordGroup):
         """
         return self._current_browser().set_window_size(width, height)
 
+    def get_window_position(self):
+        """Returns current window position as `x` then `y`.
+
+        Example:
+        | ${x} | ${y}= | Get Window Position |
+        """
+        position = self._current_browser().get_window_position()
+        return position['x'], position['y']
+
+    def set_window_position(self, x, y):
+        """Sets the position `x` and `y` of the current window to the specified values.
+
+        Example:
+        | Set Window Size | ${1000} | ${0}       |
+        | ${x} | ${y}= | Get Window Position |
+        | Should Be Equal | ${x}      | ${1000}   |
+        | Should Be Equal | ${y}      | ${0}      |
+        """
+        return self._current_browser().set_window_position(x, y)
+
     def select_frame(self, locator):
         """Sets frame identified by `locator` as current frame.
 
@@ -263,17 +290,24 @@ class _BrowserManagementKeywords(KeywordGroup):
         self._current_browser().switch_to_frame(element)
 
     def select_window(self, locator=None):
-        """Selects the window found with `locator` as the context of actions.
+        """Selects the window matching locator and return previous window handle.
+
+        locator: any of name, title, url, window handle, excluded handle's list, or special words.
+        return: either current window handle before selecting, or None if no current window.
 
         If the window is found, all subsequent commands use that window, until
         this keyword is used again. If the window is not found, this keyword fails.
-        
+
         By default, when a locator value is provided,
         it is matched against the title of the window and the
         javascript name of the window. If multiple windows with
         same identifier are found, the first one is selected.
 
-        Special locator `main` (default) can be used to select the main window.
+        There are some special locators for searching target window:
+        string 'main' (default): select the main window;
+        string 'self': only return current window handle;
+        string 'new': select the last-indexed window assuming it is the newest opened window
+        window list: select the first window not in given list (See 'List Windows' to get the list)
 
         It is also possible to specify the approach Selenium2Library should take
         to find a window by specifying a locator strategy:
@@ -289,7 +323,16 @@ class _BrowserManagementKeywords(KeywordGroup):
         | Title Should Be | Popup Title |
         | Select Window |  | | # Chooses the main window again |
         """
-        self._window_manager.select(self._current_browser(), locator)
+        try:
+            return self._current_browser().get_current_window_handle()
+        except NoSuchWindowException:
+            pass
+        finally:
+            self._window_manager.select(self._current_browser(), locator)
+
+    def list_windows(self):
+        """Return all current window handles as a list"""
+        return self._current_browser().get_window_handles()
 
     def unselect_frame(self):
         """Sets the top frame as the current frame."""
@@ -335,7 +378,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         """Logs and returns the entire html source of the current page or frame.
 
         The `loglevel` argument defines the used log level. Valid log levels are
-        `WARN`, `INFO` (default), `DEBUG`, `TRACE` and `NONE` (no logging).
+        WARN, INFO (default), DEBUG, and NONE (no logging).
         """
         source = self.get_source()
         self._log(source, loglevel.upper())
@@ -412,7 +455,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         There are several `Wait ...` keywords that take timeout as an
         argument. All of these timeout arguments are optional. The timeout
         used by all of them can be set globally using this keyword.
-        See `introduction` for more information about timeouts.
+        See `Timeouts` for more information about timeouts.
 
         The previous timeout value is returned by this keyword and can
         be used to set the old value back later. The default timeout
@@ -433,26 +476,26 @@ class _BrowserManagementKeywords(KeywordGroup):
         """Sets Selenium 2's default implicit wait in seconds and
         sets the implicit wait for all open browsers.
 
-        From selenium 2 function 'Sets a sticky timeout to implicitly 
+        From selenium 2 function 'Sets a sticky timeout to implicitly
             wait for an element to be found, or a command to complete.
             This method only needs to be called one time per session.'
 
         Example:
         | ${orig wait} = | Set Selenium Implicit Wait | 10 seconds |
         | Perform AJAX call that is slow |
-        | Set Selenium Implicit Wait | ${orig wait} | 
+        | Set Selenium Implicit Wait | ${orig wait} |
         """
         old_wait = self.get_selenium_implicit_wait()
         self._implicit_wait_in_secs = robot.utils.timestr_to_secs(seconds)
         for browser in self._cache.get_open_browsers():
             browser.implicitly_wait(self._implicit_wait_in_secs)
         return old_wait
-    
+
 
     def set_browser_implicit_wait(self, seconds):
         """Sets current browser's implicit wait in seconds.
 
-        From selenium 2 function 'Sets a sticky timeout to implicitly 
+        From selenium 2 function 'Sets a sticky timeout to implicitly
             wait for an element to be found, or a command to complete.
             This method only needs to be called one time per session.'
 
@@ -491,38 +534,38 @@ class _BrowserManagementKeywords(KeywordGroup):
 
 
     def _make_ff(self , remote , desired_capabilites , profile_dir):
-        
+
         if not profile_dir: profile_dir = FIREFOX_PROFILE_DIR
         profile = webdriver.FirefoxProfile(profile_dir)
         if remote:
-            browser = self._create_remote_web_driver(webdriver.DesiredCapabilities.FIREFOX  , 
+            browser = self._create_remote_web_driver(webdriver.DesiredCapabilities.FIREFOX  ,
                         remote , desired_capabilites , profile)
         else:
             browser = webdriver.Firefox(firefox_profile=profile)
         return browser
-    
+
     def _make_ie(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.Ie, 
+        return self._generic_make_browser(webdriver.Ie,
                 webdriver.DesiredCapabilities.INTERNETEXPLORER, remote, desired_capabilities)
 
     def _make_chrome(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.Chrome, 
+        return self._generic_make_browser(webdriver.Chrome,
                 webdriver.DesiredCapabilities.CHROME, remote, desired_capabilities)
 
     def _make_opera(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.Opera, 
+        return self._generic_make_browser(webdriver.Opera,
                 webdriver.DesiredCapabilities.OPERA, remote, desired_capabilities)
 
     def _make_phantomjs(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.PhantomJS, 
+        return self._generic_make_browser(webdriver.PhantomJS,
                 webdriver.DesiredCapabilities.PHANTOMJS, remote, desired_capabilities)
 
     def _make_htmlunit(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.Remote, 
+        return self._generic_make_browser(webdriver.Remote,
                 webdriver.DesiredCapabilities.HTMLUNIT, remote, desired_capabilities)
 
     def _make_htmlunitwithjs(self , remote , desired_capabilities , profile_dir):
-        return self._generic_make_browser(webdriver.Remote, 
+        return self._generic_make_browser(webdriver.Remote,
                 webdriver.DesiredCapabilities.HTMLUNITWITHJS, remote, desired_capabilities)
 
     def _make_android(self , remote , desired_capabilities , profile_dir):
@@ -538,9 +581,9 @@ class _BrowserManagementKeywords(KeywordGroup):
                 webdriver.DesiredCapabilities.SAFARI, remote, desired_capabilities)
 
     def _generic_make_browser(self, webdriver_type , desired_cap_type, remote_url, desired_caps):
-        '''most of the make browser functions just call this function which creates the 
+        '''most of the make browser functions just call this function which creates the
         appropriate web-driver'''
-        if not remote_url: 
+        if not remote_url:
             browser = webdriver_type()
         else:
             browser = self._create_remote_web_driver(desired_cap_type,remote_url , desired_caps)
@@ -574,4 +617,3 @@ class _BrowserManagementKeywords(KeywordGroup):
             desired_capabilities[key.strip()] = value.strip()
 
         return desired_capabilities
-    
